/**@file
 * ConnectSettings.m
 * @brief Connect setting class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/07.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "ConnectSettings.h"
#import "Base64.h"
#import "MacAddress.h"
#import "Des.h"
#import "Encoder.h"

@implementation ConnectSettings

// properties.
@synthesize m_description;
@synthesize m_url;
@synthesize m_userId;
@synthesize m_password;
@synthesize m_timeout;

/**
 * format into document.
 * @param[in] templateString tenplate string given by caller.
 * @return formatted string.
 */
- (NSString *)formatTemplate:(NSString *)templateString
{
	const NSUInteger timeout = m_timeout;
	return [NSString stringWithFormat:templateString, [Encoder encodeXmlString:m_description], [Encoder encodeXmlString:m_url],
			[Encoder encodeXmlString:m_userId], [Encoder encodeXmlString:[self encryptPassword]], timeout];
}

/**
 * parse.
 * @param[in] elements XML elements stack.
 * @param[in] attributes XML attributes.
 * @return YES if success, NO if failed.
 */
- (BOOL)parse:(NSArray *)elements withDictionary:(NSDictionary *)attributes
{
	if ([elements count] == 2) {
		if ([@"settings" compare:[elements objectAtIndex:1]] == NSOrderedSame) {
			[self setDescription:[Encoder decodeXmlString:[attributes objectForKey:@"description"]]];
			[self setUrl:[Encoder decodeXmlString:[attributes objectForKey:@"url"]]];
			[self setUserId:[Encoder decodeXmlString:[attributes objectForKey:@"userid"]]];
			[self setPassword:[Encoder decodeXmlString:[ConnectSettings decryptPassword:[attributes objectForKey:@"password"]]]];
			NSString *const timeout = [attributes objectForKey:@"timeout"];
			m_timeout = [timeout floatValue];
			return YES;
		}
	}
	return NO;
}

/**
 * encrypt password.
 * @return encrypted password.
 */
- (NSString *)encryptPassword
{
	NSString *const macAddress = [[MacAddress primaryMacAddress] substringToIndex:8];
	//NSLog(@"encryptPassword mac address = %@", macAddress);

	NSData *const encrypted = [Des encryptString:m_password encoding:NSUTF8StringEncoding key:macAddress];
	NSData *const encryptPassword = [Base64 encode:encrypted];
	NSString *const passEnc = [[[NSString alloc] initWithData:encryptPassword encoding:NSUTF8StringEncoding] autorelease];
	return passEnc;
}
	

/**
 * decrypt password.
 * @param[in] encrypted Encrypted password.
 * @return decrypted string.
 */
+ (NSString *)decryptPassword:(NSString *)encrypted
{
	NSString *const macAddress = [[MacAddress primaryMacAddress] substringToIndex:8];
	//NSLog(@"decryptPassword mac address = %@", macAddress);

	NSData *const dataEncoded = [Base64 decode:[encrypted dataUsingEncoding:NSUTF8StringEncoding]];
	NSString *const decrypted = [Des decryptToString:dataEncoded encoding:NSUTF8StringEncoding key:macAddress];
	return decrypted;
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_description = @"";
		[m_description retain];
		m_url = @"";
		[m_url retain];
		m_userId = @"";
		[m_userId retain];
		m_password = @"";
		[m_password retain];
		m_timeout = 120.0;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	[m_description release];
	[m_url release];
	[m_userId release];
	[m_password release];
	[super dealloc];
}

@end
