/**@file
 * Des.h
 * @brief DES encryption/decryption class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/09/15.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "Des.h"
#import <openssl/des.h>

static unsigned char cbc_iv[8] = {'D', 'E', 'S', 'C', 'R', 'Y', 'P', 'T'};

@implementation Des

// Encrypt from string.
+ (NSData *)encryptString:(NSString *)str encoding:(NSStringEncoding)encoding key:(NSString *)key
{
	NSData *const encodedString = [str dataUsingEncoding:encoding];
	return [Des encryptData:encodedString key:key];
}

// Encrypt data.
+ (NSData *)encryptData:(NSData *)data key:(NSString *)key
{
	// key must contains only ascii chararacters and can divide with 8.
	const char *const keyString = [key cStringUsingEncoding:NSISOLatin1StringEncoding];
	const size_t stLen = strlen(keyString);
	if ((stLen % 8 != 0) || (stLen > 32)) {
		// sorry, 32byte length is maximum so far.
		return nil;
	}

	// allocate key with length.
	NSData *const localKey = [NSData dataWithBytes:keyString length:stLen];
	DES_cblock *const keyBlock = (DES_cblock *) [localKey bytes];
	DES_set_odd_parity(keyBlock);

	DES_key_schedule schedule;
	DES_set_key_checked(keyBlock, &schedule);

	// encrypted NSData.
	NSMutableData *const encrypted = [[[NSMutableData alloc] initWithLength:[data length]] autorelease];

	// Encrypt
	int encSize = 0;
	DES_cfb64_encrypt((unsigned char*) [data bytes], (unsigned char*) [encrypted bytes],
					  [data length], &schedule, keyBlock, &encSize, DES_ENCRYPT);

	//NSLog(@"encrypted = %@", encrypted);
	return encrypted;
}

// Encrypt from string with CBC.
+ (NSData *)encryptCbcString:(NSString *)str encoding:(NSStringEncoding)encoding key:(NSString *)key
{
	NSData *const encodedString = [str dataUsingEncoding:encoding];
	return [Des encryptCbcData:encodedString key:key];
}

// Encrypt data with CBC.
+ (NSData *)encryptCbcData:(NSData *)data key:(NSString *)key
{
	// key must contains only ascii chararacters and can divide with 8.
	const char* keyString = [key cStringUsingEncoding:NSISOLatin1StringEncoding];
	const size_t stLen = strlen(keyString);
	if ((stLen % 8 != 0) || (stLen > 32)) {
		// sorry, 32byte length is maximum so far.
		return nil;
	}

	// allocate key with length.
	NSData *const localKey = [NSData dataWithBytes:keyString length:stLen];
	DES_cblock *const keyBlock = (DES_cblock*) [localKey bytes];
	DES_set_odd_parity(keyBlock);
	DES_key_schedule schedule;
	if (DES_set_key_checked(keyBlock, &schedule) != 0) {
		return nil;
	}

	// prepare IV
	DES_cblock iv3;
	memcpy(&iv3, cbc_iv, sizeof(cbc_iv));

	// encrypted NSData.
	NSUInteger toEncLen = [data length];
	NSUInteger encLen = toEncLen / 8;
	encLen += (toEncLen % 8 != 0) ? 1 : 0;
	encLen *= 8;
	NSMutableData* encrypted = [[[NSMutableData alloc] initWithLength:encLen + 1] autorelease];

	// encrypt.
	const unsigned char *const dataToEncrypt = (const unsigned char *) [data bytes];
	DES_ncbc_encrypt(dataToEncrypt, (unsigned char *) [encrypted bytes], [data length], &schedule, &iv3, DES_ENCRYPT);
	unsigned char *const dataEncrypted = (unsigned char *) [encrypted bytes];
	dataEncrypted[encLen] = '\0';
	[encrypted setLength:encLen];

	return encrypted;
}

+ (NSData *)decryptData:(NSData *)data key:(NSString *)key
{
	// key must contains only ascii chararacters and can divide with 8.
	const char *const keyString = [key cStringUsingEncoding:NSISOLatin1StringEncoding];
	const size_t stLen = strlen(keyString);
	if ((stLen % 8 != 0) || (stLen > 32)) {
		// sorry, 32byte length is maximum so far.
		return nil;
	}

	// allocate key with length.
	NSData *const localKey = [NSData dataWithBytes:keyString length:stLen];
	DES_cblock *const keyBlock = (DES_cblock*) [localKey bytes];
	DES_set_odd_parity(keyBlock);
	
	DES_key_schedule schedule;
	DES_set_key_checked(keyBlock, &schedule);

	// decrypted NSData.
	NSMutableData *const decrypted = [[[NSMutableData alloc] initWithLength:[data length]] autorelease];

	// Decrypt
	int decSize = 0;
	DES_cfb64_encrypt((unsigned char *) [data bytes], (unsigned char *) [decrypted bytes],
					  [data length], &schedule, keyBlock, &decSize, DES_DECRYPT);

	//NSLog(@"decrypted = %@", decrypted);
	return decrypted;
}

// Decrypt data with CBC.
+ (NSData *)decryptCbcData:(NSData *)data key:(NSString *)key
{
	// key must contains only ascii chararacters and can divide with 8.
	const char *const keyString = [key cStringUsingEncoding:NSISOLatin1StringEncoding];
	const size_t stLen = strlen(keyString);
	if ((stLen % 8 != 0) || (stLen > 32)) {
		// sorry, 32byte length is maximum so far.
		return nil;
	}
	
	// allocate key with length.
	NSData *const localKey = [NSData dataWithBytes:keyString length:stLen];
	DES_cblock *const keyBlock = (DES_cblock*) [localKey bytes];
	DES_set_odd_parity(keyBlock);
	DES_key_schedule schedule;
	if (DES_set_key_checked(keyBlock, &schedule) != 0) {
		return nil;
	}
	
	// prepare IV
	DES_cblock iv3;
	memcpy(&iv3, cbc_iv, sizeof(cbc_iv));
	
	// decrypted NSData.
	NSMutableData *const decrypted = [[[NSMutableData alloc] initWithLength:[data length]] autorelease];
	
	// Decrypt
	const unsigned char *const dataToDecrypt = (const unsigned char*) [data bytes];
	//NSLog(@"decrypt data length = %d", [data length]);
	//for (NSUInteger i = 0; i < [data length]; i++) {
	//	NSLog(@"0x%02x,", (int) dataToDecrypt[i]);
	//}
	DES_ncbc_encrypt(dataToDecrypt, (unsigned char *) [decrypted bytes], [data length], &schedule, &iv3, DES_DECRYPT);
	
	//NSLog(@"decrypted = %@", decrypted);
	return decrypted;
}

// Decrypt string.
+ (NSString *)decryptToString:(NSData *)data encoding:(NSStringEncoding)encoding key:(NSString *)key
{
	NSData *const decryptedString = [Des decryptData:data key:key];
	return [[[NSString alloc] initWithData:decryptedString encoding:encoding] autorelease];
}	

// Decrypt to string with CBC.
+ (NSString *)decryptCbcToString:(NSData *)data encoding:(NSStringEncoding)encoding key:(NSString *)key
{
	NSData *decryptedString = [Des decryptCbcData:data key:key];
	return [[[NSString alloc] initWithData:decryptedString encoding:encoding] autorelease];
}

@end
