/**@file
 * FileDisplay.m
 * @brief Displays remote file's attirbutes.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/08.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "FileDisplay.h"
#import "FileDisplayCollection.h"
#import "MyUtil.h"

#define MODE_NONE				@"---"	// 0
#define MODE_EXEC				@"--x"	// 1
#define MODE_WRITE				@"-w-"	// 2
#define MODE_WRITE_EXEC			@"-wx"	// 3
#define MODE_READ				@"r--"	// 4
#define MODE_READ_EXEC			@"r-x"	// 5
#define MODE_READ_WRITE			@"rw-"	// 6
#define MODE_ALL				@"rwx"	// 7

static NSString *const g_modeStrings[] = { MODE_NONE, MODE_EXEC, MODE_WRITE, MODE_WRITE_EXEC, MODE_READ, MODE_READ_EXEC, MODE_READ_WRITE, MODE_ALL };

// month struct.
typedef struct tagMONTH_VALUE {
	NSString *name;
	NSUInteger number;
} MONTH_VALUE;
static const MONTH_VALUE g_months[] = { { @"Jan", 1 }, { @"Feb", 2 }, { @"Mar", 3 }, { @"Apr", 4 }, { @"May", 5 }, { @"Jun", 6 }, { @"Jul", 7 }, { @"Aug", 8 }, { @"Sep", 9 }, { @"Oct", 10 }, { @"Nov", 11 }, { @"Dec", 12 } };

@implementation FileDisplay

// properties.
@synthesize m_name;
@synthesize m_date;
@synthesize m_owner;
@synthesize m_group;
//@synthesize m_size;
@synthesize m_mode;
@synthesize m_isDirectory;
@synthesize m_isLink;
@synthesize m_parentSortKey;
@synthesize m_parentAscending;

/**
 * get file mode string.
 * @return mode in "-rwx"
 */
- (NSString *)fileModeString
{
	const NSUInteger others = m_mode % 100 % 10;
	const NSUInteger group = (m_mode % 100 - others) / 10;
	const NSUInteger owner = (m_mode - group - others) / 100;
	unichar uc = '\0';
	if (m_isDirectory == NO) {
		if (m_isLink == NO) {
			uc = '-';
		}
		else {
			uc = 'l';
		}
	}
	else {
		uc = 'd';
	}
	return [NSString stringWithFormat:@"%c%@%@%@", uc, g_modeStrings[owner], g_modeStrings[group], g_modeStrings[others]];
}

/**
 * initialize.
 * @param[in] ftpResult FTP result string.
 * @return self
 */
- (id)initWithFtpResult:(NSString *)ftpResult
{
	self = [super init];
	if (self) {
		m_isLink = NO;
		// file or directory.
		const unichar uc = [ftpResult characterAtIndex:0];
		if (uc == 'd') {
			m_isDirectory = YES;	// directory.
		}
		else {
			m_isDirectory = NO;	// file.
			if (uc == 'l') {
				m_isLink = YES;
			}
			else if (uc == 'D') {
				m_isLink = YES;
				m_isDirectory = YES;	// link to directory.
			}
		}
		// mode.
		NSUInteger iMode[] = { 0, 0, 0 };
		NSUInteger div = 0;
		NSUInteger index = 0;
		for (NSUInteger i = 1; i < [ftpResult length] && i < 10; i++) {
			const unichar ch = [ftpResult characterAtIndex:i];
			if (ch == 'r') {
				iMode[index] |= 4;
			}
			if (ch == 'w') {
				iMode[index] |= 2;
			}
			if (ch == 'x') {
				iMode[index] |= 1;
			}
			div++;
			if (div > 2) {
				div = 0;
				index++;
			}
		}
		m_mode = (iMode[0] * 100) + (iMode[1] * 10) + iMode[2];
		// owner.
		NSString *const forOwnerSrch = [ftpResult substringFromIndex:10];
		const NSUInteger ownerStart = [self setOwner:forOwnerSrch] + 10;
		// group.
		NSString *const forGroupSrch = [ftpResult substringFromIndex:ownerStart];
		const NSUInteger groupStart = [self setGroup:forGroupSrch] + ownerStart;
		// file size.
		NSString *const forFileSizeSrch = [ftpResult substringFromIndex:groupStart];
		const NSUInteger dateStart = [self setFileSize:forFileSizeSrch] + groupStart;
		// file date.
		NSString *const forFileDateSrch = [ftpResult substringFromIndex:dateStart];
		const NSUInteger nameStart = [self setFileDate:forFileDateSrch] + dateStart;
		// file name.
		m_name = [ftpResult substringFromIndex:nameStart + 1];
		[m_name retain];
	}

	return self;
}

/**
 * get object count.
 * @return always 0.
 */
- (NSUInteger)count
{
	return 0;
}

/**
 * set owner.
 * @param[in] ftpResult FTP result string.
 * @return next start position.
 */
- (NSUInteger)setOwner:(NSString *)ftpResult
{
	m_owner = nil;
	NSUInteger pos = 0;

	// space.
	for (NSUInteger i = 0; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// non space.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	// space.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// owner name.
	const NSUInteger posStart = pos;
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	m_owner = [ftpResult substringWithRange:NSMakeRange(posStart, pos - posStart)];
	[m_owner retain];
	
	return pos;
}

/**
 * set group.
 * @param[in] ftpResult FTP result string.
 * @return next start position.
 */
- (NSUInteger)setGroup:(NSString *)ftpResult
{
	m_group = nil;
	NSUInteger pos = 0;

	// space.
	for (NSUInteger i = 0; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// group name.
	const NSUInteger posStart = pos;
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	m_group = [ftpResult substringWithRange:NSMakeRange(posStart, pos - posStart)];
	[m_group retain];
	
	return pos;
}

/**
 * set file size.
 * @param[in] ftpResult FTP result string.
 * @return next start position.
 */
- (NSUInteger)setFileSize:(NSString *)ftpResult
{
	m_size = 0;
	NSUInteger pos = 0;
	
	// space.
	for (NSUInteger i = 0; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// file size.
	const NSUInteger posStart = pos;
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	NSString *const size = [ftpResult substringWithRange:NSMakeRange(posStart, pos - posStart)];
	m_size = [size intValue];
	
	return pos;
}

/**
 * set file date.
 * @param[in] ftpResult FTP result string.
 * @return next start position.
 */
- (NSUInteger)setFileDate:(NSString *)ftpResult
{
	m_date = nil;
	NSUInteger pos = 0;

	// space.
	for (NSUInteger i = 0; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// file date.
	const NSUInteger posStart = pos;
	// non space for month.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	// space.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// non space for day.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}
	// space.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch != ' ') {
			break;
		}
		pos++;
	}
	// non space for time or year.
	for (NSUInteger i = pos; i < [ftpResult length]; i++) {
		const unichar ch = [ftpResult characterAtIndex:i];
		if (ch == ' ') {
			break;
		}
		pos++;
	}

	m_date = [ftpResult substringWithRange:NSMakeRange(posStart, pos - posStart)];
	[m_date retain];
	
	return pos;
}

/**
 * compare.
 * @param[in] object Compare target object.
 * @return YES if same, NO if not.
 */
- (BOOL)isEqualFile:(id)object
{
	//if ([object class] != [self class]) {
	//	return NO;
	//}

	if (m_isDirectory != [object isDirectory]) {
		return NO;
	}
	if (m_isLink != [object isLink]) {
		return NO;
	}
	if (m_size !=  [object fileSize]) {
		return NO;
	}
	if (m_mode !=  [object fileMode]) {
		return NO;
	}
	
	if (([m_name compare:[object fileName]] != NSOrderedSame) || ([m_date compare:[object fileDate]] != NSOrderedSame) ||
		([m_owner compare:[object fileOwner]] != NSOrderedSame) || ([m_group compare:[object fileGroup]] != NSOrderedSame)) {
		return NO;
	}
	return YES;
}

/**
 * copy function.
 * @param[in] fileDisplay the copy source.
 */
- (void)copyFrom:(FileDisplay *)fileDisplay
{
	[self setFileName:[fileDisplay fileName]];
	[m_date release];
	m_date = [fileDisplay fileDate];
	[m_date retain];
	[m_owner release];
	m_owner = [fileDisplay fileOwner];
	[m_owner retain];
	[m_date retain];
	[m_group release];
	m_group = [fileDisplay fileGroup];
	[m_group retain];
	m_size = [fileDisplay fileSize];
	m_mode = [fileDisplay fileMode];
	m_isDirectory = [fileDisplay isDirectory];
	m_isLink = [fileDisplay isLink];
}

/**
 * copare function.
 * @param[in] pair compare target.
 */
- (NSComparisonResult)compare:(id)pair
{
	NSComparisonResult res = NSOrderedSame;
	if ([m_parentSortKey compare:@"Owner"] == NSOrderedSame) {
		// compare with owner.
		res = [m_owner compare:[pair fileOwner]];
	}
	else if ([m_parentSortKey compare:@"Group"] == NSOrderedSame) {
		// compare with group.
		res = [m_group compare:[pair fileGroup]];
	}
	else if ([m_parentSortKey compare:@"Date"] == NSOrderedSame) {
		// compare with date.
		const NSUInteger myDate = [self integerFileDate];
		const NSUInteger pairDate = [pair integerFileDate];
		if (myDate < pairDate) {
			res = NSOrderedAscending;
		}
		else if (myDate > pairDate) {
			res = NSOrderedDescending;
		}
	}
	else if ([m_parentSortKey compare:@"Size"] == NSOrderedSame) {
		// compare size.
		const NSUInteger pairSize = [pair fileSize];
		if (m_size < pairSize) {
			res = NSOrderedAscending;
		}
		else if (m_size > pairSize) {
			res = NSOrderedDescending;
		}
	}
	else if ([m_parentSortKey compare:@"Mode"] == NSOrderedSame) {
		// compare mode.
		const NSUInteger pairMode = [pair fileMode];
		if (m_mode < pairMode) {
			res = NSOrderedAscending;
		}
		else if (m_mode > pairMode) {
			res = NSOrderedDescending;
		}
	}
	else {
		// compare name.
		res = [m_name compare:[pair fileName]];
	}

	if (m_parentAscending == NO) {
		if (res == NSOrderedAscending) {
			res = NSOrderedDescending;
		}
		else if (res == NSOrderedDescending) {
			res = NSOrderedAscending;
		}
	}
	return res;
}

/**
 * date in NSUInteger.
 * @return date in NSUInteger.
 */
- (NSUInteger)integerFileDate
{
	// month.
	NSUInteger month = 0;
	NSString *const monName = [m_date substringToIndex:3];
	for (NSUInteger i = 0; i < COUNT_OF(g_months, MONTH_VALUE); i++) {
		if ([monName compare:g_months[i].name] == NSOrderedSame) {
			month = g_months[i].number;
			break;
		}
	}
	// day.
	NSString *const dayName = [m_date substringWithRange:NSMakeRange(4, 2)];
	const NSUInteger day = [dayName intValue];
	// time or year.
	NSString *const timeName = [m_date substringFromIndex:7];
	NSUInteger hour = 0;
	NSUInteger minute = 0;
	NSUInteger year = 0;
	if ([timeName rangeOfString:@":"].length == 0) {
		// year
		year = [timeName intValue];
	}
	else {
		// year as current year.
		NSTimeZone *const sysTz = [NSTimeZone systemTimeZone];
		NSDateFormatter *const dateFormatter = [[NSDateFormatter alloc] init];
		dateFormatter.dateFormat = @"yyyy";
		[dateFormatter setTimeZone:sysTz];
		NSString *const yearName = [dateFormatter stringFromDate:[NSDate date]];
		year = [yearName intValue];
		[dateFormatter release];
		// time.
		NSString *const hourName = [timeName substringToIndex:2];
		NSString *const minuteName = [timeName substringFromIndex:3];
		hour = [hourName intValue];
		minute = [minuteName intValue];
	}

	return (year * 100000000) + (month * 1000000) + (day * 10000) + (hour * 100) + minute;
}

/**
 * file name if link strip link target.
 * @return file name without link target.
 */
- (NSString *)fileTitle
{
	NSString* fileName = m_name;
	if (m_isLink) {
		fileName = [fileName substringToIndex:[fileName rangeOfString:@" -> "].location];
	}
	return fileName;
}

/**
 * file size.
 * @return file size;
 */
- (unsigned long long)fileSize
{
	return m_size;
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		[self setFileName:@""];
		m_date = @"";
		[m_date retain];
		m_owner = @"";
		[m_owner retain];
		m_group = @"";
		[m_group retain];
		m_size = 0;
		m_mode = 0;
		m_isDirectory = NO;
		m_isLink = NO;
		m_parentSortKey = nil;
		m_parentAscending = YES;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	//NSLog(@"dealloc = %p, name = %@", self, m_name);
	[m_parentSortKey release];
	[m_name release];
	[m_date release];
	[m_owner release];
	[m_group release];
	[super dealloc];
}

// description.
//- (NSString *)description
//{
//	return [NSString stringWithFormat:@"%c%d %@ %@ %d %@", (m_isDirectory == NO) ? '-' : 'd', m_mode, m_owner, m_group, m_size, m_name];
//}

@end
