/**@file
 * FileDisplayController.m
 * @brief Controller class for FileDisplay.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/09.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "FileDisplayController.h"
#import "ViewController.h"
#import "ShsslClient.h"

@implementation FileDisplayController

// properties.
@synthesize m_controller;
@synthesize m_currentDirectory;
@synthesize m_processPath;

/**
 * set view controller.
 * @param[in] viewController ViewController class.
 */
- (void)setController:(id)viewController
{
	m_controller = viewController;
	[m_collection setController:viewController];
}

/**
 * before add files.
 */
- (void)invalidate
{
	[m_pathDictionary removeAllObjects];
	[m_collection setPathDictionary:m_pathDictionary];	// set path dictionary.
}

/**
 * after add files.
 */
- (void)updateAll
{
	[self setProcessPath:@""];
}

/**
 * add display line.
 * @param[in] ftpResult FTP result string.
 */
- (void)addFileDisplay:(NSString *)ftpResult
{
	FileDisplayCollection *const fdc = [m_pathDictionary objectForKey:m_processPath];
	[fdc addFileDisplay:ftpResult];
}

/**
 * add file display end.
 */
- (void)addFileDisplayEnd
{
	FileDisplayCollection *const fdc = [m_pathDictionary objectForKey:m_processPath];
	[fdc childrenSort];
}

/**
 * remove all objects.
 */
- (void)removeAllObjects
{
	[m_collection removeAllObjects];
	[self invalidate];
}

/**
 * get file view's data source.
 * @return current file view's data source.
 */
- (FileDisplayCollection *)currentFileViewDataSource
{
	//NSLog(@"cur dir = %@", m_currentDirectory);
	FileDisplayCollection *const fdc = [m_pathDictionary objectForKey:m_currentDirectory];
	return fdc;
}

/**
 * expands only current directory.
 * @param[in] outlineView Outline view to expand.
 */
- (void)expandOutlineView:(NSOutlineView *)outlineView
{
	[m_collection expandOutlineView:outlineView currentDirectory:m_currentDirectory];
}

/**
 * outline view data source handler.
 * @param[in] outlineView outline view to retrieve children.
 * @param[in] index index of child.
 * @param[in] item parent child.
 * @return new selected item.
 */
- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
{
	if (item == nil) {
		return m_collection;
	}
	else {
		return [item directoryAtIndex:index];
	}
}

/**
 * outline view data source handler.
 * @param[in] outlineView outline view to expand.
 * @param[in] item item to expand.
 * @return YES.
 */
- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	return YES;
}

/**
 * outline view data source handler.
 * @param[in] outlineView outline view of retrieveing number.
 * @param[in] item item to retrieve child count.
 * @return child count.
 */
- (NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	NSInteger dirCnt = 0;
	if (item == nil) {
		dirCnt = 1;	// root.
	}
	else {
		id client = [m_controller client];
		// is the part of current path?
		BOOL listed = NO;
		NSString *const itemDir = [item fullPath];
		const NSRange range = [m_currentDirectory rangeOfString:itemDir];
		if (range.location != 0) {
			// if not on current directory path...
			dirCnt = [item directoryCount];
			//if ((dirCnt == 0) || ([item isDummy])) {
			if (dirCnt == 0) {
				//[item setDummy:NO];
				// restrain the LIST command invoke...
				[item invalidate];
				[client list:itemDir];	// LIST call to CGI server.
				[item updateAll];
				listed = YES;	// invoked LIST call.
			}
		}

		dirCnt = [item directoryCount];

		if (listed) {
			// back to current directory.
			[client setCurrentDirectory:m_currentDirectory];
		}
	}
	return dirCnt;
}

/**
 * outline view data source handler.
 * @param[in] outlineView outline view of retrieving data.
 * @param[in] tableColumn table column of retrieving data.
 * @param[in] item item to retrieve showing data count.
 * @return showng data.
 */
- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	//NSLog(@"outlineView:objectValueForTableColumn:byItem:");

	if (item == m_collection) {
		return [item fullPath];
	}
	else {
		return [item fileName];
	}
}

/**
 * outline view delegate handler.
 * @param[in] outlineView outline view to be selected.
 * @param[in] item item to be selected.
 * @return showng data.
 */
- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
{
	[m_controller directorySelectionHasChanged:item];
	[self setCurrentDirectory:[item fullPath]];
	return YES;
}

/**
 * retrieve file display.
 * @param[in] filePath file path to retrieve.
 * @return file display object.
 */
- (id)retrieveFileDisplay:(NSString *)filePath
{
	if ([filePath compare:@"/"] == NSOrderedSame) {
		return m_collection;	// root directory.
	}

	return [m_collection fileDisplay:filePath];
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_controller = nil;
		m_currentDirectory = @"";
		[m_currentDirectory retain];
		m_processPath = @"";
		[m_processPath retain];
		m_collection = [[FileDisplayCollection alloc] init];
		[m_collection setFullPath:@"/"];
		m_pathDictionary = [[NSMutableDictionary alloc] init];
		[m_collection setPathDictionary:m_pathDictionary];	// set path dictionary.
	}
	return self;
}

// destructor.
- (void)dealloc
{
	[m_pathDictionary release];
	[m_processPath release];
	[m_collection release];
	[m_currentDirectory release];
	[super dealloc];
}

@end
