/**@file
 * FileDropTableView.m
 * @brief File drop table view class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/17
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "FileDropTableView.h"
#import "FileDisplayCollection.h"
#import "ViewController.h"
#import "MyDocument.h"
#import "MyUtil.h"

// constants.
#define PRESS_SHIFT								0x20000
#define PRESS_COMMAND							0x100000

@implementation FileDropTableView

/**
 * after XIB initialized.
 */
- (void)awakeFromNib;
{
	//NSLog(@"[FileDropTableView awakeFromNib]");

	NSTableColumn *const column = [self tableColumnWithIdentifier:@"Name"];
	[self setIndicatorImage:[NSImage imageNamed:@"NSAscendingSortIndicator"] inTableColumn:column];	// sort ascending image as default.

	[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType] ];
	[self setAllowsMultipleSelection:YES];
	[self setVerticalMotionCanBeginDrag:NO];
	m_bInDragging = NO;
}

/**
 * mouse down event handler.
 * @param[in] theEvent mouse event object.
 */
- (void)mouseDown:(NSEvent *)theEvent
{
	//NSLog(@"mouseDown:%@", theEvent);

	//
	// We need to check that promised files are supported - they are not in pre-10.2 OSs.
	//
	if ([self respondsToSelector:@selector(dragPromisedFilesOfTypes:fromRect:source:slideBack:event:)] == NO) {
		NSLog( @"Promised file drag-and-drop is not supported." );
		[super mouseDown:theEvent];
		return;
	}

	NSPoint dragPosition = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	const NSUInteger restoreDragRow = [self rowAtPoint:dragPosition];
	// double click?
	if ([theEvent clickCount] == 2) {
		// double click.
		[self mouseDoubleClicked:theEvent row:restoreDragRow];
		return;
	}
	
	[self setClickPointSelected:theEvent row:restoreDragRow];
	
	[NSEvent startPeriodicEventsAfterDelay:0.3 withPeriod:0];
	NSEvent *const newEvent = [[self window] nextEventMatchingMask:NSLeftMouseUpMask | NSLeftMouseDraggedMask];
	id ds = [self dataSource];
	
	NSRect imageLocation;
	switch ([newEvent type]) {
		case NSLeftMouseUp:
			[NSEvent stopPeriodicEvents];
			break;
		case NSLeftMouseDragged:
			[NSEvent stopPeriodicEvents];
			if ((restoreDragRow != -1) && (restoreDragRow < [ds count])) {
       			dragPosition.x -= 28;
       			dragPosition.y -= 10;
				imageLocation.origin = dragPosition;
				imageLocation.size = NSMakeSize(16, 16);
				NSString *const extension = @"docs";
				[self dragPromisedFilesOfTypes:[NSArray arrayWithObject:extension] fromRect:imageLocation source:self slideBack:YES event:theEvent];
			}
			break;
    	default:
			break;
	}
}

/**
 * double clicked.
 * @param[in] theEvent mouse event object.
 * @param[in] row clicked row index.
 */
- (void)mouseDoubleClicked:(NSEvent *)theEvent row:(NSUInteger)row
{
	id ds = [self dataSource];
	id fd = [ds displayObjectAtIndex:row];
	if ((fd != nil) && ([fd isDirectory])) {
		// directory change.
		[m_controller directoryDoubleClicked:fd];
	}
}

/**
 * Has dropped to finder.
 * @param[in] url finder URL.
 * @return nil
 */
- (NSArray *)namesOfPromisedFilesDroppedAtDestination:(NSURL *)url
{
	//NSLog(@"Dropped to: %@\n", [url path]);
	NSString *const targetDir = [url path];
	[m_controller retrieveDroppedFile:targetDir];
	return nil;
}

/**
 * call while dragging.
 * @param[in] isLocal YES from this application, No from other applications.
 * @return Copy operation.
 */
- (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)isLocal;
{
	//NSLog(@"draggingSourceOperationMaskForLocal:%@", isLocal == NO ? @"NO" : @"YES");
	if (isLocal) {
		return NSDragOperationNone;
	}
	else {
		return NSDragOperationCopy;
	}
}

/**
 * window event have to delay or not.
 * @param[in] theEvent window event.
 * @return YES.
 */
- (BOOL)shouldDelayWindowOrderingForEvent:(NSEvent *)theEvent
{
	//NSLog(@"shouldDelayWindowOrderingForEvent:%@", theEvent);
	return YES;
}

/**
 * accepts first mouse or not.
 * @param[in] theEvent mouse event.
 * @return YES.
 */
- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	//NSLog(@"acceptsFirstMouse:%@", theEvent);
	return YES;
}

/**
 * set/reset dragging.
 * @param[in] flag YES if dragging, NO if not.
 */
- (void)setInDragging:(BOOL)flag
{
	// Change display mode.
	m_bInDragging = flag;
	[self setNeedsDisplay:YES];
}

/**
 * receive file list from dragging information.
 * @param[in] info dragging information.
 * @return file list.
 */
- (NSArray *)fileListInDraggingInfo:(id <NSDraggingInfo>)info
{
    // paste board object
    NSPasteboard *poPasteBd = [info draggingPasteboard];
    // file list of draging.
    NSArray *const parrFiles = [poPasteBd propertyListForType:NSFilenamesPboardType];

    return parrFiles;
}

/**
 * notify enter dragging.
 * @param[in] sender a sender object.
 * @return dragging operation type.
 */
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
	// examine droppable or not.
	const BOOL bRet = [self acceptableObject:sender];
	if (bRet != YES) {
		return NSDragOperationNone;
	}

	// set dragging
	[self setInDragging: YES];

	return NSDragOperationCopy;
}

/**
 * current dragging object is droppable on this object or not.
 * @param[in] info dragging info.
 * @return YES if droppable, NO if not.
 */
- (BOOL)acceptableObject:(id <NSDraggingInfo>)info
{
	NSFileManager *const poFM = [NSFileManager defaultManager];
	// receive file list of dragging.
	NSArray *const parrFiles = [self fileListInDraggingInfo:info];

	for (NSString *const pstrPath in parrFiles) {
		// dragging object is acceptable or not.
		BOOL bDir = NO;
		const BOOL bRet = [poFM fileExistsAtPath: pstrPath isDirectory: &bDir];
		if ( bRet != YES ) {
			// could not be happened.
			return NO;
		}
	}
	return YES;
}

/**
 * dragging out from the field.
 * @param[in] sender a sender object.
 */
- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	[self setInDragging:NO];
}

/**
 * draw rectangle.
 * @param[in] dirtyRect rectangle to draw.
 */
- (void)drawRect:(NSRect)dirtyRect
{
	// super class method.
	[super drawRect:dirtyRect];

	if (m_bInDragging != NO) {
        // thick frame while dragging.
        [[NSColor selectedControlColor] set];
        NSFrameRectWithWidth(dirtyRect, 2.0);
    }
}

/**
 * set click point selected.
 * @param[in] theEvent user event.
 * @param[in] row new selected row.
 */
- (void)setClickPointSelected:(NSEvent *)theEvent row:(NSUInteger)row
{
	const NSUInteger modifierFlags = [theEvent modifierFlags];
	NSIndexSet *const indexSetOld = [self selectedRowIndexes];	// curent select indices.
	NSUInteger index = [indexSetOld firstIndex];
	while (index != NSNotFound) {
		if (index == row) {
			return;	// click on the already selected do nothing.
		}
		index = [indexSetOld indexGreaterThanIndex:index];
	}
	
	if (modifierFlags & PRESS_SHIFT) {
		// shift pressed.
		NSMutableIndexSet *const mutableIndex = [NSMutableIndexSet indexSet];
		const NSUInteger firstIndex = [indexSetOld firstIndex];
		const NSUInteger lastIndex = [indexSetOld lastIndex];
		if ((row >= firstIndex) && (row <= lastIndex)) {
			// between.
			for (NSUInteger i = firstIndex; i <= row; i++) {
				[mutableIndex addIndex:i];
			}
			NSUInteger index = [indexSetOld indexGreaterThanIndex:row];
			while (index != NSNotFound) {
				[mutableIndex addIndex:index];
				index = [indexSetOld indexGreaterThanIndex:index];
			}
		}
		else if (row <= firstIndex) {
			// leth than or equal to start position.
			for (NSUInteger i = row; i < firstIndex; i++) {
				[mutableIndex addIndex:i];
			}
			NSUInteger index = firstIndex;
			while (index != NSNotFound) {
				[mutableIndex addIndex:index];
				index = [indexSetOld indexGreaterThanIndex:index];
			}
		}
		else if (row >= lastIndex) {
			// greater than or equal to last position.
			NSUInteger index = firstIndex;
			while (index != NSNotFound) {
				[mutableIndex addIndex:index];
				index = [indexSetOld indexGreaterThanIndex:index];
			}
			for (NSUInteger i = lastIndex; i <= row; i++) {
				[mutableIndex addIndex:i];
			}
		}
		[self selectRowIndexes:mutableIndex byExtendingSelection:NO];
	}
	else if (modifierFlags & PRESS_COMMAND) {
		// command pressed.
		NSMutableIndexSet *const mutableIndex = [NSMutableIndexSet indexSet];
		NSUInteger index = [indexSetOld firstIndex];
		BOOL add = NO;
		while (index != NSNotFound) {
			if (index > row) {
				[mutableIndex addIndex:row];
				add = YES;
			}
			else if (index == row) {
				add = YES;
			}

			[mutableIndex addIndex:index];
			index = [indexSetOld indexGreaterThanIndex:index];
		}
		if (add == NO) {
			[mutableIndex addIndex:row];
		}
		[self selectRowIndexes:mutableIndex byExtendingSelection:NO];
	}
	else {
		NSIndexSet *const indexSet = [NSIndexSet indexSetWithIndex:row];
		[self selectRowIndexes:indexSet byExtendingSelection:NO];
	}

	//NSLog(@"mouse event = %@", theEvent);
}

/**
 * set new delagate.
 * @param[in] delagate new delagate.
 */
- (void)setDataSource:(id <NSTableViewDataSource>)aSource
{
	if ([self dataSource] != aSource) {
		[self clearColumnImages];
		// set column image.
		id dataSource = aSource;
		NSTableColumn *const column = [self tableColumnWithIdentifier:[dataSource sortKey]];
		NSString *const imageName = ([dataSource isAscending]) ? @"NSAscendingSortIndicator" : @"NSDescendingSortIndicator";
		[self setIndicatorImage:[NSImage imageNamed:imageName] inTableColumn:column];
	}

	[super setDataSource:aSource];
}

/**
 * clear columns images.
 */
- (void)clearColumnImages
{
	NSTableColumn *column = [self tableColumnWithIdentifier:@"Name"];
	[self setIndicatorImage:nil inTableColumn:column];
	column = [self tableColumnWithIdentifier:@"Owner"];
	[self setIndicatorImage:nil inTableColumn:column];
	column = [self tableColumnWithIdentifier:@"Group"];
	[self setIndicatorImage:nil inTableColumn:column];
	column = [self tableColumnWithIdentifier:@"Date"];
	[self setIndicatorImage:nil inTableColumn:column];
	column = [self tableColumnWithIdentifier:@"Size"];
	[self setIndicatorImage:nil inTableColumn:column];
	column = [self tableColumnWithIdentifier:@"Mode"];
	[self setIndicatorImage:nil inTableColumn:column];
}

/**
 * cancel edit with escape key.
 * @param[in] sender a sender object.
 */
- (void)cancelOperation:(id)sender
{
	[self abortEditing];
	[[self window] makeFirstResponder:self];
}

@end
