/**@file
 * MyDocument.m
 * @brief The document class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/07.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "MyDocument.h"
#import "ConnectSettings.h"
#import "MyUtil.h"
#import "XmlDelegater.h"

@implementation MyDocument

// properties.
@synthesize m_controller;
@synthesize m_connectionSetting;
@synthesize m_canRedoCnt;
@synthesize m_lastDirectory;

/**
 * rename menu.
 * @param[in] sender the sender object.
 */
- (IBAction)renameToMenu:(id)sender
{
	[m_controller renameToMenu:sender];
}

/**
 * chmod menu.
 * @param[in] sender the sender object.
 */
- (IBAction)chmodToMenu:(id)sender
{
	[m_controller chmodToMenu:sender];
}

/**
 * rmv menu.
 * @param[in] sender the sender object.
 */
- (IBAction)removeFileMenu:(id)sender
{
	[m_controller removeFileMenu:sender];
}

/**
 * mkdir menu.
 * @param[in] sender the sender object.
 */
- (IBAction)makeDirectoryMenu:(id)sender
{
	[m_controller makeDirectoryMenu:sender];
}

/**
 * retr menu.
 * @param[in] sender the sender object.
 */
- (IBAction)retrieveFile:(id)sender
{
	[m_controller retrieveFile:sender];
}

/**
 * stor menu.
 * @param[in] sender the sender object.
 */
- (IBAction)addFile:(id)sender
{
	[m_controller addFile:sender];
}

/**
 * reload menu.
 * @param[in] sender the sender object.
 */
- (IBAction)reloadMenu:(id)sender
{
	[m_controller reloadMenu:sender];
}

/**
 * stor menu with paste.
 * @param[in] sender the sender object.
 */
- (IBAction)paste:(id)sender
{
	[m_controller paste:sender];
}

/**
 * stor menu with paste from finder.
 * @param[in] sender the sender object.
 */
- (IBAction)pasteFromFinder:(id)sender
{
	[m_controller pasteFromFinder:sender];
}

/**
 * copy.
 * @param[in] sender the sender object.
 */
- (IBAction)copy:(id)sender
{
	[m_controller copy:sender];
}

/**
 * cut.
 * @param[in] sender the sender object.
 */
- (IBAction)cut:(id)sender
{
	[m_controller cut:sender];
}

/**
 * parent directory.
 * @param[in] sender the sender object.
 */
- (IBAction)parentDirectory:(id)sender
{
	[m_controller parentDirectory:sender];
}

/**
 * change directory.
 * @param[in] sender the sender object.
 */
- (IBAction)changeDirectory:(id)sender
{
	[m_controller changeDirectory:sender];
}

/**
 * set new connection setting.
 * @param[in] setting a new setting.
 */
- (void)setConnectionSetting:(id)setting
{
	// Undo, Redo.
	[[[self undoManager] prepareWithInvocationTarget:self] setConnectionSetting:m_connectionSetting];

	m_connectionSetting = setting;
	[self setDirty];
}

/**
 * set document dirty.
 */
- (void)setDirty
{
	if ([[self undoManager] isUndoing] != NO) {
		// doing undo.
		[self updateChangeCount:NSChangeUndone];
		m_canRedoCnt++;
	}
	else {
		[self updateChangeCount:NSChangeDone];
		if ([[self undoManager] isRedoing] != NO) {
			// doing redo.
			m_canRedoCnt--;
		}
		else {
			m_canRedoCnt = 0;
		}
	}
}

/**
 * write data.
 * @param[in] url tempoary file's URL.
 * @param[in] typeName always @"DocumentType".
 * @param[out] outError error information.
 * @return always YES.
 */
- (BOOL)writeToURL:(NSURL *)url ofType:(NSString *)typeName error:(NSError **)outError
{
	// clear undo objects.
	[[self undoManager] removeAllActions];
	m_canRedoCnt = 0;

	NSString *const template = [MyUtil templateString:@"root"];
	NSString *const content = [m_connectionSetting formatTemplate:template];
	[content writeToURL:url atomically:YES encoding:NSUTF8StringEncoding error:nil];
	return YES;
}

/**
 * XML parsing.
 * @param[in] xmlData file content.
 * @param[out] outError error information.
 * @return YES if success, NO if failed.
 */
- (BOOL)parseXml:(NSData *)xmlData error:(NSError **)outError
{
	BOOL ret = NO;
	NSXMLParser *parser = nil;
	XmlDelegater *delegater = nil;
	@try {
		// initialize parser.
		parser = [[NSXMLParser alloc] initWithData:xmlData];
		delegater = [[XmlDelegater alloc] init];
		[delegater setTarget:self];
		[parser setDelegate:delegater];

		// start parsing.
		[parser parse];

		// success.
		ret = YES;
	}
	@catch (NSException *e) {
		//NSLog(@"parseXml = %@", [e reason]);
		*outError = [NSError errorWithDomain:NSOSStatusErrorDomain code:unimpErr userInfo:[e userInfo]];
	}
	@finally {
		[parser release];
		[delegater release];
	}

	return ret;
}

/**
 * Add element call from XML parser.
 * @param[in] elemArray Element array, the last one is the current element.
 * @param[in] dict attribute dictionary.
 * @return YES if success, NO if failed.
 */
- (BOOL)addElement:(NSArray *)elemArray withDictionary:(NSDictionary *)dict
{
	const NSUInteger count = [elemArray count];
	if (count == 0) {
		return NO;
	}
	if ([@"shsslft" compare:[elemArray objectAtIndex:0]] != NSOrderedSame) {
		return NO;
	}

	// from [1] to compare.
	if (count == 2) {
		NSString* const rootName = [elemArray objectAtIndex:1];
		// csvinfo targets this object.
		if ([rootName compare:@"settings"] == NSOrderedSame) {
			// settings.
			return [m_connectionSetting parse:elemArray withDictionary:dict];
		}
	}
	else {
		// the oldest version does not care about the file version.
		return YES;
	}

	return YES;	// new version ignore, just read the well known elements.
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		// Add your subclass-specific initialization here.
		// If an error occurs here, send a [self release] message and return nil.
		m_connectionSetting = [[ConnectSettings alloc] init];
		m_canRedoCnt = 0;
		[self setLastDirectory:[MyUtil documentDirectory]];
	}
	return self;
}

// getting XIB name.
- (NSString *)windowNibName
{
	// Override returning the nib file name of the document
	// If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
	return @"MyDocument";
}

// After XIB loaded.
- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
	[super windowControllerDidLoadNib:aController];
	// Add any code here that needs to be executed once the windowController has loaded the document's window.
}

// write data handler.
- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
	// Insert code here to write your document to data of the specified type. If the given outError != NULL, ensure that you set *outError when returning nil.
	// You can also choose to override -fileWrapperOfType:error:, -writeToURL:ofType:error:, or -writeToURL:ofType:forSaveOperation:originalContentsURL:error: instead.
	// For applications targeted for Panther or earlier systems, you should use the deprecated API -dataRepresentationOfType:. In this case you can also choose to override -fileWrapperRepresentationOfType: or -writeToFile:ofType: instead.
    if (outError != NULL) {
		*outError = [NSError errorWithDomain:NSOSStatusErrorDomain code:unimpErr userInfo:NULL];
	}
	return nil;
}

// read data handler.
- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
	const BOOL ret = [self parseXml:data error:outError];
	// Insert code here to read your document from the given data of the specified type.  If the given outError != NULL, ensure that you set *outError when returning NO.
	// You can also choose to override -readFromFileWrapper:ofType:error: or -readFromURL:ofType:error: instead. 
	// For applications targeted for Panther or earlier systems, you should use the deprecated API -loadDataRepresentation:ofType. In this case you can also choose to override -readFromFile:ofType: or -loadFileWrapperRepresentation:ofType: instead.
	if ( outError != NULL ) {
		*outError = [NSError errorWithDomain:NSOSStatusErrorDomain code:unimpErr userInfo:NULL];
	}
	return ret;
}

// destructor.
- (void)dealloc
{
	[m_lastDirectory release];
	[m_connectionSetting release];
	[super dealloc];
}

@end
