/**@file
 * MyUtil.m
 * @brief Utility function class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/07.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "MyUtil.h"

// constant.
#define MULTI_BYTE_SPACE							0x3000

// characters do not use in path. use in isValidHttpProtocol()
static NSString *const g_doNotForPath = @"<>&*?\"\\";

@implementation MyUtil

/**
 * Message box function.
 * @param[in] text messege box text
 * @return user clicked button id.
 */
+ (NSInteger)messageBox:(NSString *)text
{
	return [MyUtil messageBox:text withTitle:nil];
}

/**
 * Message box function.
 * @param[in] text messege box text
 * @param[in] title messege box title
 * @return user clicked button id.
 */
+ (NSInteger)messageBox:(NSString *)text withTitle:(NSString *)title
{
	NSString *myTitle = @"ShSSLFT";
	if (title != nil) {
		myTitle = title;
	}
	NSAlert *const alert = [NSAlert alertWithMessageText:myTitle
								defaultButton:@"OK"
								alternateButton:nil
								otherButton:nil
								informativeTextWithFormat:text];
	return [alert runModal];
}

/**
 * Message box function, international versions..
 * @param[in] text messege box text
 * @return user clicked button id.
 */
+ (NSInteger)messageBoxInternational:(NSString *)text
{
	NSString *const intlText = NSLocalizedString(text, text);
	return [MyUtil messageBox:intlText withTitle:nil];
}

/**
 * Message box function, international versions..
 * @param[in] text messege box text
 * @param[in] title messege box title
 * @return user clicked button id.
 */
+ (NSInteger)messageBoxInternational:(NSString *)text withTitle:(NSString *)title
{
	NSString *const intlText = NSLocalizedString(text, text);
	NSString *const intlTitle = NSLocalizedString(title, title);
	return [MyUtil messageBox:intlText withTitle:intlTitle];
}

+ (NSInteger)askYesNo:(NSString *)text
{
	return [self askYesNo:text withTitle:nil];
}

+ (NSInteger)askYesNo:(NSString *)text withTitle:(NSString *)title
{
	NSString *myTitle = @"ShSSLFT";
	if (title != nil) {
		myTitle = title;
	}
	NSAlert *const alert = [NSAlert alertWithMessageText:myTitle
									 defaultButton:NSLocalizedString(@"Yes", @"Yes")
								   alternateButton:NSLocalizedString(@"No", @"No")
									   otherButton:nil
						 informativeTextWithFormat:text];
	return [alert runModal];
}

/**
 * Get user document directory.
 * @return user document directory.
 */
+ (NSString *)documentDirectory
{
	NSArray *const paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);    
	return [paths objectAtIndex:0];  
}

/**
 * Focus to NSTextField
 * @param[in] target text field.
 * @param[in] dialog owner dialog class.
 */
+ (void)focusTo:(NSTextField *)control onDialog:(id)dialog
{
	[control selectText:dialog];
	[[control currentEditor] setSelectedRange:NSMakeRange(0, -1)];
}

/**
 * Get template string.
 * @param path template file path name.
 * @return template string.
 */
+ (NSString *)templateString:(NSString *)path
{
	NSString *const tplt = [[NSBundle mainBundle] pathForResource:path ofType:@"template"];
	NSData *const data = [NSData dataWithContentsOfFile:tplt];
	return [[NSString alloc] initWithData:data encoding:NSUTF16StringEncoding];
}

/**
 * trim string.
 * @param[in] string before trim.
 * @return string after trim.
 */
+ (NSString *)trim:(NSString *)str
{
	const NSInteger len = [str length];
	NSInteger head = 0;
	for (head = 0; head < len; head++) {
		unichar c = [str characterAtIndex:head];
		if ((c != '\r') && (c != '\n') && (c != '\t') && (c != ' ') && (c != MULTI_BYTE_SPACE)) {
			break;
		}
	}
	if (head == len) {
		return @"";	// only space characters has come.
	}
	NSInteger tail = 0;
	for (tail = len - 1; tail > -1; tail--) {
		unichar c = [str characterAtIndex:tail];
		if ((c != '\r') && (c != '\n') && (c != '\t') && (c != ' ') && (c != MULTI_BYTE_SPACE)) {
			break;
		}
	}

	if (head == 0) {
		if (tail == len - 1) {
			return str;	// no trim.
		}
		else {
			return [str substringToIndex:tail + 1];	// trim tail.
		}
	}
	else {
		if (tail == len - 1) {
			return [str substringFromIndex:head];	// trim head.
		}
		else {
			return [str substringWithRange:NSMakeRange(head, tail - head + 1)];	// trim head and tail.
		}

	}
}

/**
 * create current date or time.
 @param[in] fmt format string.
 * @return current date formatter.
 */
+ (NSString *)currentDateTime:(NSString *)fmt;
{
	NSDate *const date = [NSDate date];
	NSDateFormatter *const formatter = [[NSDateFormatter alloc] init];
	[formatter setTimeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
	[formatter setDateFormat:fmt];
	NSString *const dateTimeStr = [formatter stringFromDate:date];
	[formatter release];

	return dateTimeStr;
}

/**
 * is integer string or not.
 * @param[in] str string to examine integer or not.
 * @return YES if an integer, NO if not an integer.
 */
+ (BOOL)isIntegerString:(NSString *)str
{
	if ([str length] == 0) {
		return NO;	// empty string.
	}
	NSUInteger startPos = 0;
	if ([str characterAtIndex:0] == '-') {
		if ([str length] == 1) {
			return NO;	// @"-"
		}
		startPos++;
	}
	for (NSUInteger i = startPos; i < [str length]; i++) {
		unichar c = [str characterAtIndex:i];
		if ((c < '0') || (c > '9')) {
			return NO;
		}
	}
	// all digits.
	return YES;
}

/**
 * http, https URL validation.
 * @param[in] url URL string to validate http/https.
 * @return YES if valid http/https URL, NO if not.
 */
+ (BOOL)isValidHttpProtocol:(NSString *)url
{
	NSString *tmpUrl;
	const NSRange rngHttp = [url rangeOfString:@"http://"];
	const NSRange rngHttps = [url rangeOfString:@"https://"];
	if ((rngHttp.location != 0) && (rngHttps.location != 0)) {
		return NO;
	}
	if (rngHttp.location == 0) {
		tmpUrl = [url substringFromIndex:rngHttp.length];
	}
	else {
		tmpUrl = [url substringFromIndex:rngHttps.length];
	}
	
	// divide with slash.
	NSArray *const sla = [tmpUrl componentsSeparatedByString:@"/"];
	if ([[sla objectAtIndex:0] length] == 0) {
		// no host name.
		return NO;
	}
	// host name with port number?
	NSArray *const pNo = [[sla objectAtIndex:0] componentsSeparatedByString:@":"];
	if ([pNo count] > 2) {
		return NO;
	}
	else if ([pNo count] == 2) {
		// must be a port number.
		if ([[pNo objectAtIndex:1] length] == 0) {
			return NO;	// no port number specified.
		}
		NSUInteger portNo = [[pNo objectAtIndex:1] intValue];
		if (portNo > 65535) {
			// do not exceed 65535.
			return NO;
		}
	}
	// validate host name.
	NSString *const hostName = [pNo objectAtIndex:0];
	// divide with dot.
	NSArray *const dot = [hostName componentsSeparatedByString:@"."];
	BOOL hasNoNum = NO;
	for (NSString *const locDom in dot) {
		if ([locDom length] == 0) {
			return NO;
		}
		for (NSUInteger i = 0; i < [locDom length]; i++) {
			unichar c = [locDom characterAtIndex:i];
			if ((c >= 'a') && (c <= 'z')) {
				// lower case.
				hasNoNum = YES;
			}
			else {
				if ((c >= 'A') && (c <= 'Z')) {
					// upper case.
					hasNoNum = YES;
				}
				else {
					if ((c == '-') || (c == '_')) {
						// hifn or underscore.
						hasNoNum = YES;
					}
					else {
						if ((c < '0') || (c > '9')) {
							return NO;
						}
						// numeric.
					}
				}
			}
		}
	}
	
	// if numeric only, ip address.
	if (hasNoNum == NO) {
		if ([dot count] != 4) {
			return NO;
		}
		for (NSString *const locDom in dot) {
			NSUInteger iVal = [locDom intValue];
			if (iVal > 255) {
				return NO;
			}
		}
	}
	
	// validation path.
	BOOL first = YES;
	for (NSString *const path in sla) {
		if (first == NO) {
			for (NSUInteger i = 0; i < [path length]; i++) {
				unichar c = [path characterAtIndex:i];
				for (NSUInteger j = 0; j < [g_doNotForPath length]; j++) {
					if ([g_doNotForPath characterAtIndex:j] == c) {
						return NO;	// found the character prohibited to use.
					}
				}
			}
		}
		else {
			first = NO;
		}
	}
	
	// success.
	return YES;
}

/**
 * ask local file is directory or not.
 * @param[in] path local file path.
 * @return YES for directory, NO if physical file.
 */
+ (BOOL)isDirectory:(NSString *)path
{
	NSFileManager *const fileManager = [NSFileManager defaultManager];
	BOOL isDir = NO;
	[fileManager fileExistsAtPath:path isDirectory:&isDir];
	return isDir;
}

/**
 * format file name intarnational.
 * @param[in] format internationalize format.
 * @param[in] file file name.
 * @return formatted string
 */
+ (NSString *)internationalFormat:(NSString *)format file:(NSString *)file
{
	return [NSString stringWithFormat:NSLocalizedString(format, format), file];
}

@end
