/**@file
 * ShsslClient.m
 * @brief Client class, Server is CGI.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/08.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "ShsslClient.h"
#import "ViewController.h"
#import "MyDocument.h"
#import "ConnectSettings.h"
#import "Encoder.h"
#import "FileDisplayCollection.h"
#import "FileDisplayController.h"
#import "XmlResponseScanner.h"

// constants.
static NSString* const DATA_BOUNDARY = @"0xKhTmLbOuNdArY";

@implementation ShsslClient

// properties.
@synthesize m_currentDirectory;
@synthesize m_connect;
@synthesize m_controller;
@synthesize m_command;
@synthesize m_param0;
@synthesize m_param1;
@synthesize m_commandResult;

/**
 * connect to CGI.
 * @param[in] connect YES if connect, NO if disconnect.
 * @return YES if operation success, NO if operation fail.
 */
- (BOOL)connect:(BOOL)connect
{
	m_connect = connect;
	[self setCommand:COMMAND_CONN];
	m_finish = NO;
	if (connect == NO) {
		[self setCurrentDirectory:@"~"];
		m_commandResult = YES;
		return YES;	// discconect has nothing to do any more.
	}

	m_paramCount = 0;
	const BOOL ret = [self send];

	if (ret == NO) {
		m_connect = NO;
	}

	return ret;
}

/**
 * LIST command.
 * @return YES if operation success, NO if operation fail.
 */
- (BOOL)list:(NSString *)currentDir
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCurrentDirectory:currentDir];
	[self setCommand:COMMAND_LIST];

	m_paramCount = 0;
	return [self send];
}

/**
 * REN command.
 * @param[in] from old name.
 * @param[in] to new name.
 * @return YES if success, NO if not.
 */
- (BOOL)renameFileFrom:(NSString *)from to:(NSString *)to
{
	if (m_connect == NO) {
		return NO;
	}
	
	[self setCommand:COMMAND_REN];
	[self setParam0:from];
	[self setParam1:to];

	m_paramCount = 2;
	return [self send];
}

/**
 * CHMOD command.
 * @param[in] pathName file path.
 * @param[in] mode mode in 3 digits.
 * @return YES if success, NO if not.
 */
- (BOOL)chmod:(NSString *)pathName mode:(NSString *)mode
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCommand:COMMAND_CHMOD];
	[self setParam0:pathName];
	[self setParam1:mode];

	m_paramCount = 2;
	return [self send];
}

/**
 * remove directory.
 * @param[in] pathName directory path to remove.
 * @return YES if success, NO if not.
 */
- (BOOL)rmd:(NSString *)pathName
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCommand:COMMAND_RMD];
	[self setParam0:pathName];

	m_paramCount = 1;
	return [self send];
}

/**
 * remove file.
 * @param[in] pathName file path to remove.
 * @return YES if success, NO if not.
 */
- (BOOL)rm:(NSString *)pathName
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCommand:COMMAND_RM];
	[self setParam0:pathName];

	m_paramCount = 1;
	return [self send];
}

/**
 * make directory.
 * @param[in] pathName directory name to create.
 * @return YES if success, NO if not.
 */
- (BOOL)mkd:(NSString *)pathName
{
	if (m_connect == NO) {
		return NO;
	}
	
	[self setCommand:COMMAND_MKD];
	[self setParam0:pathName];
	
	m_paramCount = 1;
	return [self send];
}

/**
 * retr file.
 * @param[in] pathName file name to retrieve.
 * @param[out] content file content.
 * @return YES if success, NO if not.
 */
- (BOOL)retr:(NSString *)pathName content:(NSMutableData *)content
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCommand:COMMAND_RETR];
	[self setParam0:pathName];
	m_contentReceiver = content;	// set receiver 

	m_paramCount = 1;
	const BOOL ret = [self send];
	m_contentReceiver = nil;
	return ret;
}

/**
 * stor file.
 * @param[in] pathName file name to retrieve.
 * @param[in] content file content to store.
 * @return YES if success, NO if not.
 */
- (BOOL)stor:(NSString *)pathName content:(NSData *)content
{
	if (m_connect == NO) {
		return NO;
	}

	[self setCommand:COMMAND_STOR];
	[self setParam0:pathName];
	m_contentSender = content;	// set receiver 

	m_paramCount = 0;	// use m_param0 but 0 params.
	const BOOL ret = [self send];
	m_contentReceiver = nil;
	return ret;
}

/**
 * send.
 * @return YES if success, NO if not.
 */
- (BOOL)send
{
	m_commandResult = NO;
	
	[self releaseLock];
	[self resetLock];
	
	// create thread.
	[NSThread detachNewThreadSelector:@selector(threadProc:) toTarget:self withObject:nil];
	
	// waiting for thread end.
	const BOOL ret = [self waitForComplete];
	
	return ret;
}

/**
 * release lock object.
 */
- (void)releaseLock
{
	[m_lock release];
	m_lock = nil;
}

/**
 * Wait for complete.
 * @return YES if success, NO if failed.
 */
- (BOOL)waitForComplete
{
	[m_lock lockWhenCondition:1];
	[m_lock unlock];
	return m_result;
}

/**
 * reset lock.
 */
- (void)resetLock
{
	m_finish = NO;
	m_lock = [[NSConditionLock alloc] initWithCondition:0];
}

/**
 * complete signal.
 */
- (void)downloadCompleted
{
	[m_lock lock];
	[m_lock unlockWithCondition:[m_lock condition] + 1];
}

/**
 * create connection.
 * @param[in] request request object.
 * @return created connection.
 */
- (NSURLConnection *)createConnection:(NSURLRequest *)request
{
	// only call at child thread context.
	m_data = [[[NSMutableData data] retain] autorelease];
	
	// create connection.
	[m_conn release];	// just in case...
	m_conn = [[NSURLConnection alloc] initWithRequest:request delegate:self];
	return m_conn;
}

/**
 * wait loop of child thread.
 */
- (void)currentRunLoop
{
	do {
		[[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
	} while (m_finish == NO);
}

/**
 * connection started, or redirected.
 * @param[in] connection connection object.
 * @param[in] response response object.
 */
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	//NSLog(@"[HttpSender connection:didReceiveResponse:]");
	// find encoding.
	[self findEncodeString:response];
	
	[m_data setLength:0];
}

/**
 * data has come.
 * @param[in] connection connection object.
 * @param[in] data sent from server.
 */
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	//NSLog(@"[HttpSender connection:didReceiveData:]");
	[m_data appendData:data];
}

/**
 * all data has been sent
 * @param[in] connection connection object.
 */
- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	//NSLog(@"[HttpSender connectionDidFinishLoading:]");
	m_result = YES;
	m_finish = YES;
}

/**
 * error occurred.
 * @param[in] connection connection object.
 * @param[in] error error information.
 */
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	//NSLog(@"[HttpSender connection:didFailWithError:]");
	m_finish = YES;
}

/**
 * thread procedure.
 * @param[in] anArgument arugment from parent thread.
 */
- (void)threadProc:(id)anArgument
{
	// Asynchronus call.
	NSAutoreleasePool *const pool = [[NSAutoreleasePool alloc] init];
	NSURLRequest *const request = [self createRequest:anArgument];
	if (request) {
		// create connection.
		[self createConnection:request];
		if (m_conn) {
			[self currentRunLoop];
			[m_conn release];
			m_conn = nil;

			// reflect results.
			[self  reflectResults];
		}
	}

	[pool drain];
	m_data = nil;
	[self downloadCompleted];
}

/**
 * create request.
 * @param[in] params parameter array.
 * @return url request.
 */
- (NSURLRequest *)createRequest:(NSArray *)params
{
	id doc = [m_controller owner];
	NSString *const baseUrl = [[doc connectionSetting] urlString];

	BOOL hasContent = NO;
	// create URL.
	NSURL *const url = [NSURL URLWithString:baseUrl];
	// create a request.
	NSMutableURLRequest* const request = [NSMutableURLRequest requestWithURL:url];
	[request setValue:@"text/html,application/xml;q=9.0,*/*;9=0.8" forHTTPHeaderField:@"Accept"];
	[request setValue:@"Shift_JIS,utf-8;q=0.8,*q=0.7" forHTTPHeaderField:@"Accept-Charset"];
	[request setValue:@"gzip,deflate" forHTTPHeaderField:@"Accept-Encoding"];
	[request setValue:@"ja,en-us;q=0.7;q=0.3" forHTTPHeaderField:@"Accept-Language"];
	if ([m_command compare:COMMAND_STOR] != NSOrderedSame) {
		// normal application/x-www-form-urlencoded.
		[request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
	}
	else {
		// upload file content.
		[request setValue:[NSString stringWithFormat:@"multipart/form-data; boundary=%@", DATA_BOUNDARY] forHTTPHeaderField:@"Content-Type"];
		hasContent = YES;
	}
	[request setValue:@"http://www.comona.co.jp/" forHTTPHeaderField:@"Referer"];
	[request setValue:@"ShSSLFT/0.1 (Macintosh; U; Wintel Mac OS X 10.6; ja-JP-mac; rv:1.9.2.10) Gacko/20101008 ShSSLFT/0.1.0" forHTTPHeaderField:@"User-Agent"];
	[request setCachePolicy:NSURLRequestUseProtocolCachePolicy];
	[request setTimeoutInterval:[[doc connectionSetting] timeout]];

	// post
	[request setHTTPMethod:@"POST"];
	NSData *data = nil;
	if (hasContent == NO) {
		data = [self createRequestContent];
	}
	else {
		data = [self createUploadRequestContent];
	}
	[request setHTTPBody:data];
	
	return request;
}

/**
 * create request content.
 * @return request content data.
 */
- (NSData *)createRequestContent
{
	id setting = [[m_controller owner] connectionSetting];

	// create content string.
	NSString *contentStr = nil;
	switch (m_paramCount) {
		case 0:
			contentStr = [NSString stringWithFormat:@"user=%@&password=%@&dir=%@&cmd=%@", [Encoder encodeHttpString:[setting userId]],
						  [Encoder encodeHttpString:[setting password]],
						  [Encoder encodeHttpString:m_currentDirectory],
						  [Encoder encodeHttpString:m_command]];
			break;
		case 1:
			contentStr = [NSString stringWithFormat:@"user=%@&password=%@&dir=%@&cmd=%@%&@&param0=%@",
						  [Encoder encodeHttpString:[setting userId]],
						  [Encoder encodeHttpString:[setting password]],
						  [Encoder encodeHttpString:m_currentDirectory],
						  [Encoder encodeHttpString:m_command],
						  [Encoder encodeHttpString:m_param0]];
			break;
		case 2:
			contentStr = [NSString stringWithFormat:@"user=%@&password=%@&dir=%@&cmd=%@%&@&param0=%@&param1=%@",
						  [Encoder encodeHttpString:[setting userId]],
						  [Encoder encodeHttpString:[setting password]],
						  [Encoder encodeHttpString:m_currentDirectory],
						  [Encoder encodeHttpString:m_command],
						  [Encoder encodeHttpString:m_param0],
						  [Encoder encodeHttpString:m_param1]];
			break;
		default:
			@throw [NSException exceptionWithName:@"ShsslClient" reason:@"Unknown command" userInfo:nil];
	}

	// create data from string.
	NSData *const data = [contentStr dataUsingEncoding:NSASCIIStringEncoding];
	return data;
}

/**
 * create request content for store file.
 * @return request content data.
 */
- (NSData *)createUploadRequestContent
{
	id setting = [[m_controller owner] connectionSetting];

	NSMutableData *const postBody = [NSMutableData data];
	// user.
	[postBody appendData:[[NSString stringWithFormat:@"--%@\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"user\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:[setting userId]] dataUsingEncoding:NSUTF8StringEncoding]];
	// password.
	[postBody appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"password\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:[setting password]] dataUsingEncoding:NSUTF8StringEncoding]];
	// current directory.
	[postBody appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"dir\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:m_currentDirectory] dataUsingEncoding:NSUTF8StringEncoding]];
	// command.
	[postBody appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"cmd\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:m_command] dataUsingEncoding:NSUTF8StringEncoding]];
	// file.
	[postBody appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"file\"; filename=\"%@\"\r\n", m_param0] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:[[NSString stringWithString:@"Content-Type: application/octet-stream\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[postBody appendData:m_contentSender];
	[postBody appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",DATA_BOUNDARY] dataUsingEncoding:NSUTF8StringEncoding]];
	
	// return post body.
	return postBody;
}

/**
 * find server encode.
 * @param[in] response The response sent from server.
 */
- (void)findEncodeString:(NSURLResponse *)response
{
	// find encoding.
	NSString *const encoding = [[response textEncodingName] lowercaseString];
	if ([encoding isEqualToString:@"shift_jis"] || [encoding isEqualToString:@"sjis"]) {
		m_htmlEnc = NSShiftJISStringEncoding;
	}
	else if ([encoding isEqualToString:@"utf-8"] || [encoding isEqualToString:@"utf8"]) {
		m_htmlEnc = NSUTF8StringEncoding;
	}
	else if ([encoding isEqualToString:@"euc-jp"] || [encoding isEqualToString:@"euc_jp"]) {
		m_htmlEnc = NSJapaneseEUCStringEncoding;
	}
	else {
		// default!
		m_htmlEnc = NSUTF8StringEncoding;
	}
}

/**
 * reflect results.
 */
- (void)reflectResults
{
	XmlResponseScanner *scanner = nil;
	NSString *xmlString = nil;
	@try {
		xmlString = [[NSString alloc] initWithData:m_data encoding:m_htmlEnc];
		//NSLog(@"%@", xmlString);
		scanner = [[XmlResponseScanner alloc] init];
		[scanner setResultTarget:m_controller];	// set ViewController as result target.
		[scanner setShsslClient:self];	// to recieve command result.
		[scanner setContentReceiver:m_contentReceiver];	// contentnreceiver.
		[scanner setXmlString:xmlString];	// XML content.

		// scan content.
		[scanner scan];
	}
	@catch (NSException *e) {
		if (m_connect == NO) {
			[self performSelectorOnMainThread:@selector(disconnectedMainThread:) withObject:nil waitUntilDone:NO];
		}
		m_result = NO;
	}
	@finally {
		[scanner release];
		[xmlString release];
	}
}

/**
 * notify to main thread connection is disconnected.
 */
- (void)disconnectedMainThread:(id)some
{
	[m_controller disconnected];
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_conn = nil;
		m_lock = nil;
		m_finish = NO;
		m_data = nil;
		m_result = NO;
		m_connect = NO;
		m_htmlEnc = NSUTF8StringEncoding;
		m_foundChars = [[NSMutableString alloc] init];
		[self setCurrentDirectory:@"~"];
		[self setCommand:@""];
		m_param0 = nil;
		m_param1 = nil;
		m_paramCount = 0;
		m_commandResult = NO;
		m_contentReceiver = nil;
		m_contentSender = nil;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	[m_param1 release];
	[m_param0 release];
	[m_foundChars release];
	[m_conn release];
	[m_lock release];
	[m_data release];	// just in case...
	[m_command release];
	[m_currentDirectory release];
	[super dealloc];
}

@end
